/*
 * Light-weight Reliable Multicast
 *
 * lrm_api.h - Public lrm api definitions.
 * Authors: Yoann Janvier, Gal Roualland.
 * See COPYING file for license information.
 */

#ifndef LRM_API_H
#define LRM_API_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/time.h>
#include <sys/types.h>
 
struct lrm_session;

/* prototypes */
struct lrm_session* lrm_create_session(unsigned long add, unsigned short port, unsigned short ttl, unsigned long uid);
unsigned long lrm_hostid();
void  lrm_destroy_session (struct lrm_session *);

void  lrm_join(struct lrm_session *);
void  lrm_leave(struct lrm_session *); 

void  lrm_set_drop_probability(struct lrm_session *s, float percentage);
void  lrm_set_debug (int debug);

int   lrm_socket(struct lrm_session *s);
int   lrm_isidle(struct lrm_session *s);
int   lrm_can_send(struct lrm_session *s);
int   lrm_can_recv(struct lrm_session *s);
int   lrm_send(struct lrm_session *s, void * data, int len);
int   lrm_recv(struct lrm_session *s, void * data, int len);
int   lrm_recvfrom(struct lrm_session *s, void * data, int len, struct sockaddr *from, int *fromlen);

/* event loop prototypes */
int   lrm_process_one_event(struct lrm_session *s);
void  lrm_process_timers(struct lrm_session *s);
void  lrm_process_messages(struct lrm_session *s);
struct timeval lrm_get_timeval(struct lrm_session *s);
  
#define LRM_SET_TIMER_SENDACK    1
#define LRM_SET_TIMER_RESEND     2
#define LRM_SET_TIMER_PING       3
#define LRM_SET_WAIT_RESEND      4
#define LRM_SET_MAX_PING         5
#define LRM_SET_MAX_RESEND       6
#define LRM_SET_WAIT_FOR_RESENT  7
#define LRM_SET_WIN_SIZE         8
#define LRM_SET_FORCE_ACK        9
#define LRM_SET_WAIT_ANNOUNCE   10
#define LRM_SET_OPTIONS         11
#define LRM_GET_OPTIONS         12

#define LRM_STRICT               1
#define LRM_BLOCK                2
#define LRM_DELIVER_ON_LEAVE     4
#define LRM_DEFAULT              (LRM_BLOCK|LRM_DELIVER_ON_LEAVE)
  
int   lrm_set(struct lrm_session *s, int param, int value);
  
#ifdef __cplusplus
}
#endif

#endif LRM_API_H
