/*
 * Light-weight Reliable Multicast
 *
 * lrm_list.c - Semi-generic double linked list.
 * Author: Gal Roualland.
 * See COPYING file for license information.
 */

#ifndef LRM_LIST_H
#define LRM_LIST_H

#ifndef LRM_SUCCESS
#define LRM_SUCCESS 1
#endif
#ifndef LRM_FAILURE
#define LRM_FAILURE 0
#endif

#ifdef __cplusplus
extern "C" {
#endif

struct lrm_cell;

typedef struct lrm_cell {
  void * data;
  struct lrm_cell * previous;
  struct lrm_cell * next;
} lrm_cell;

typedef struct lrm_list_s {
  lrm_cell * first;
  lrm_cell * last;
  int count;
} lrm_list_s;

typedef lrm_list_s * lrm_list;
typedef lrm_cell * lrm_iterator;

lrm_list     lrm_list_new();
void         lrm_list_free(lrm_list);

#ifdef LRM_LIST_NO_MACROS
int          lrm_list_count(lrm_list);
int          lrm_list_isempty(lrm_list);
lrm_iterator lrm_list_begin(lrm_list);
lrm_iterator lrm_list_end(lrm_list);
lrm_iterator lrm_list_next(lrm_iterator);
lrm_iterator lrm_list_previous(lrm_iterator);
void*        lrm_list_content(lrm_iterator);
int          lrm_list_insert_first(lrm_list, void *);
int          lrm_list_insert_last(lrm_list, void *);
#else
#define      lrm_list_isempty(lst)            ((lst->first==lst->last) && (lst->last==NULL))
#define      lrm_list_count(lst)              (lst->count)
#define      lrm_list_begin(lst)              (lst->first)
#define      lrm_list_end(lst)                (lst->last)
#define      lrm_list_next(iter)              (iter ? iter->next : NULL)
#define      lrm_list_previous(iter)          (iter ? iter->previous : NULL)
#define      lrm_list_content(iter)           (iter ? iter->data : NULL)
#define      lrm_list_insert_first(lst, data) (lrm_list_insert_before(lst, lst->first, data))
#define      lrm_list_insert_last(lst, data)  (lrm_list_insert_after(lst, lst->last, data))
#endif

int          lrm_list_insert_before(lrm_list, lrm_iterator, void *);
int          lrm_list_insert_after(lrm_list, lrm_iterator, void *);
lrm_iterator lrm_list_delete(lrm_list, lrm_iterator);

#ifdef __cplusplus
}
#endif

  
#endif
