/*
 * Light-weight Reliable Multicast
 *
 * lrm_messages.h - lrm messages definitions.
 * Authors: Yoan Janvier, Gal Roualland.
 * See COPYING file for license information.
 */

#ifndef LRM_MESSAGES_H
#define LRM_MESSAGES_H

#define LRM_DIFFUSION 0
#define LRM_ACK       1
#define LRM_PING      2
#define LRM_PONG      3
#define LRM_JOIN      4
#define LRM_LEAVE     5

#ifdef __cplusplus
extern "C" { 
#endif

void * lrm_create_message(struct lrm_session *s, lrm_type stype);
void *lrm_create_diffusion_message (struct lrm_session *s, lrm_seqno seqno, char * data, int data_length, int *message_length);
void *lrm_create_ack_message(struct lrm_session *s, lrm_seqno seqno,int *message_length);
void lrm_send_message_mcast(struct lrm_session *s, void * message, int len);
void lrm_send_message_ucast(struct lrm_session *s, unsigned long addr, void * message, int len);
void lrm_send_ack (struct lrm_session * s, struct lrm_user * u);
void lrm_announce(struct lrm_session *);

int lrm_save_sent_message (struct lrm_session *s, void *data, int len);

void lrm_give_message(struct lrm_session *s, struct lrm_user *u, struct lrm_recvd_message * m);
void lrm_give_user_queue(struct lrm_session *s, struct lrm_user *user);

void lrm_free_sent_message(struct lrm_sent_message * m);
void lrm_free_sent_messages(struct lrm_session * s) ;

void lrm_free_recvd_message(struct lrm_recvd_message * m);
void lrm_free_recvd_messages(struct lrm_session * s, struct lrm_user * u);

#ifdef __cplusplus
}
#endif
#endif
