/*
 * Light-weight Reliable Multicast
 *
 * mcat.c - Simple mcast app. send what's on stdin, and display on stdout what's received.
 * Authors: Yoan Janvier, Gal Roualland.
 * See COPYING file for license information.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
#include <arpa/inet.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
# include <sys/time.h>
# else
#  include <time.h>
# endif
#endif
#if STDC_HEADERS
#include <string.h>
#else
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
char *strchr (), *strrchr ();
# ifndef HAVE_MEMCPY
#  define memcpy(d, s, n) bcopy ((s), (d), (n))
#  define memmove(d, s, n) bcopy ((s), (d), (n))
# endif
#endif

#include <stdlib.h>
#include <stdio.h>

#define BUFSIZE 4096

int goon = 1;

RETSIGTYPE end(int sig) {
  goon = 0;
}

void recvdata(void *data, u_int len, struct sockaddr * remoteExtremity, int remoteExtremityLen, void * extradata) {
  *((int *) extradata) += len;
  write(STDOUT_FILENO, data, len);
}

int main(int argc, char ** argv) {
  unsigned char buf[BUFSIZE+1];
  fd_set fdsin;
  int lu;
  struct timeval startr;
  struct timeval endr;
  struct timeval starts;
  struct timeval ends;
  unsigned long addr;
  unsigned short portm;
  unsigned short ttl;
  struct hostent * host;
  u_int recvsize = 0;
  u_int sendsize = 0;
  struct ip_mreq mreq;
  struct sockaddr_in sin;
  int sock;
  char opt;
  int starteds = 0, startedr = 0;
  int len;
  double secr, secs;
  
  if (argc!=4) {
    fprintf(stderr,"usage: %s address port ttl\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  addr = inet_addr(argv[1]);
  if(addr==-1) {
    host = gethostbyname(argv[1]);
    if (!host) {
      fprintf(stderr, "%s: invalid address.\n", argv[0]);
      exit(EXIT_FAILURE);
    }
    memcpy(&addr, *host->h_addr_list, sizeof(addr));
  }

  portm = htons(atoi(argv[2]));
  ttl = atoi(argv[3]);

  sock = socket(AF_INET, SOCK_DGRAM, 0);
  if (sock<0) {
    perror("socket");
    exit(EXIT_FAILURE);
  }

  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY; // addr;
  sin.sin_port = portm;
  
  if(bind(sock, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
    perror("bind");
    exit(EXIT_FAILURE);
  }
  
  mreq.imr_interface.s_addr = INADDR_ANY;
  mreq.imr_multiaddr.s_addr = addr; 
  if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *) &mreq, sizeof(mreq)) < 0) {
    perror ("setsockopt");
    exit(EXIT_FAILURE);
  }
  
  opt = ttl;
  if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (char *)&opt, sizeof(opt)) < 0) {
    perror("setsockopt");
    exit(EXIT_FAILURE);
  }
    
  opt = 0;
  if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&opt, sizeof(opt)) < 0) {
    perror("setsockopt");
    exit(EXIT_FAILURE);
  }

  signal(SIGINT, end);
  while (goon) {
    FD_ZERO (&fdsin);
    FD_SET (0, &fdsin);
    FD_SET (sock, &fdsin);    
    
    if (select(sock+1, &fdsin, NULL, NULL, NULL)<0) {
      perror("select");
      goon = 0;
      continue;
    }
        
    if(FD_ISSET(0, &fdsin)) {
      if (!starteds) {
	gettimeofday(&starts, NULL);
	starteds = 1;
      }
      lu = read(0, buf+1, BUFSIZE);
      sendsize += lu+1;
      
      sin.sin_port = portm;
      sin.sin_family = AF_INET;
      sin.sin_addr.s_addr = addr;
      if (lu>0) {
	gettimeofday(&ends, NULL);
	if (sendto(sock, buf, lu+1, 0, (struct sockaddr *) &sin, sizeof(struct sockaddr_in))<0) {
	  perror("sendto");
	  exit(EXIT_FAILURE);
	}
      } else {
	for (goon=0;goon<5;goon++)
	  sendto(sock, buf, 1, 0, (struct sockaddr *) &sin, sizeof(struct sockaddr_in));
	goon = 0;
      }
    } 

    if(FD_ISSET(sock, &fdsin)) {
      if (!startedr) {
	gettimeofday(&startr, NULL);
	startedr = 1;
      }
      len = sizeof(struct sockaddr_in);
      lu = recvfrom(sock, buf, BUFSIZE, 0, (struct sockaddr *) &sin, &len);
      if (lu<0) {
	perror("recvfrom");
	exit(EXIT_FAILURE);
      }
      if (lu>1) {
	recvdata(buf+1, lu-1, (struct sockaddr *) &sin, sizeof(struct sockaddr_in),
		 (void *) &recvsize);
	gettimeofday(&endr, NULL);
      } else
	goon = 0;
	recvsize++;
    }
  }     

  secs = ((double) ends.tv_sec - (double) starts.tv_sec) +  
    ((double) ends.tv_usec - (double) starts.tv_usec) / (double) 1000000;

  secr = ((double) endr.tv_sec - (double) startr.tv_sec) +  
    ((double) endr.tv_usec - (double) startr.tv_usec) / (double) 1000000;

  if (secs==0) secs = 0.1;
  if (secr==0) secr = 0.1;

  fprintf(stderr, "sent %d bytes [%g kb/s], received %d bytes [%g kb/s].\n",
	  sendsize, (double) sendsize / (secs * 1024),
	  recvsize, (double) recvsize / (secr * 1024));
  
  return(EXIT_SUCCESS);
}


